package com.ruoyi.bst.order.service.impl;

import java.util.Objects;

import org.springframework.stereotype.Service;

import com.ruoyi.bst.app.domain.AppVO;
import com.ruoyi.bst.area.domain.AreaVO;
import com.ruoyi.bst.channel.domain.ChannelVO;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.domain.enums.DeviceStatus;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.domain.bo.OrderCreateBO;
import com.ruoyi.bst.order.domain.bo.OrderEndBO;
import com.ruoyi.bst.order.domain.bo.OrderPayBO;
import com.ruoyi.bst.order.domain.dto.OrderCreateDTO;
import com.ruoyi.bst.order.domain.dto.OrderPayDTO;
import com.ruoyi.bst.order.domain.enums.OrderStatus;
import com.ruoyi.bst.order.domain.vo.OrderInParkingVO;
import com.ruoyi.bst.order.domain.vo.OrderPrePriceVO;
import com.ruoyi.bst.order.service.OrderValidator;
import com.ruoyi.bst.order.utils.OrderUtil;
import com.ruoyi.bst.suit.domain.SuitVO;
import com.ruoyi.common.core.domain.vo.UserVO;
import com.ruoyi.common.utils.MathUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;

@Service
public class OrderValidatorImpl implements OrderValidator{

    @Override
    public void validate(OrderCreateBO bo) {
        ServiceUtil.assertion(bo == null, "参数错误");

        OrderCreateDTO dto = bo.getDto();
        ServiceUtil.assertion(dto == null, "参数错误");

        // 设备
        DeviceVO device = bo.getDevice();
        ServiceUtil.assertion(device == null, "ID为%s的车辆不存在", dto.getDeviceId());
        ServiceUtil.assertion(CollectionUtils.isEmptyElement(device.getModelSuitIds()), "ID为%s的车辆无可用套餐", dto.getDeviceId());
        ServiceUtil.assertion(!DeviceStatus.canCreateOrder().contains(device.getStatus()), "ID为%s的车辆当前状态不可下单", dto.getDeviceId());
        ServiceUtil.assertion(!MathUtils.smallerThan(device.getRemainingPower(), device.getAreaUndercharge()), "ID为%s的车辆电量不足%s%%，暂时无法使用", dto.getDeviceId(), device.getAreaUndercharge());

        // 套餐
        SuitVO suit = bo.getSuit();
        ServiceUtil.assertion(suit == null, "ID为%s的套餐不存在", dto.getSuitId());
        ServiceUtil.assertion(!device.getModelSuitIds().contains(suit.getId()), "ID为%s的套餐不可在ID为%s的车辆使用", dto.getSuitId(), dto.getDeviceId());
        ServiceUtil.assertion(suit.getSeconds() == null || suit.getSeconds() <= 0, "ID为%s的套餐时长不能为0或小于0", dto.getSuitId());

        // 用户
        UserVO user = bo.getUser();
        ServiceUtil.assertion(user == null, "ID为%s的用户不存在", dto.getUserId());
        // TODO 实名认证

        // 价格
        OrderPrePriceVO prePrice = bo.getPrePrice();
        ServiceUtil.assertion(prePrice == null, "价格计算失败");
        // 是否和前端传输的价格一致
        boolean priceEquals = OrderUtil.isEquals(prePrice, dto.getPrice());
        ServiceUtil.assertion(!priceEquals, "当前价格已发生变化，请刷新后重新下单");
        
    }

    @Override
    public boolean canPay(OrderVO order, Long userId) {
        return isUser(order, userId);
    }

    @Override
    public boolean canEnd(OrderVO order, Long userId) {
        return isUser(order, userId);
    }

    // 是否是下单用户
    private boolean isUser(OrderVO order, Long userId) {
        return order != null && userId != null && Objects.equals(order.getUserId(), userId);
    }

    @Override
    public void validate(OrderPayBO bo) {
        ServiceUtil.assertion(bo == null, "参数错误");

        OrderPayDTO dto = bo.getDto();
        ServiceUtil.assertion(dto == null, "参数错误");

        // 订单
        OrderVO order = bo.getOrder();
        ServiceUtil.assertion(order == null, "编号为%s的订单不存在", dto.getNo());
        ServiceUtil.assertion(!OrderStatus.canPay().contains(order.getStatus()), "编号为%s的订单当前状态不可支付", dto.getNo());
        
        // 应用
        AppVO app = bo.getApp();
        ServiceUtil.assertion(app == null, "ID为%s的应用不存在", dto.getAppId());


        // 渠道
        ChannelVO channel = bo.getChannel();
        ServiceUtil.assertion(channel == null, "ID为%s的渠道不存在", dto.getChannelId());
        ServiceUtil.assertion(channel.getEnabled() == null || !channel.getEnabled(), "ID为%s的渠道暂不可用", channel.getChannelId());
        ServiceUtil.assertion(channel.getAppIds() == null || !channel.getAppIds().contains(app.getId()), "ID为%s的渠道不支持ID为%s的应用", channel.getChannelId(), app.getId());
        
        // 设备
        DeviceVO device = bo.getDevice();
        ServiceUtil.assertion(device == null, "ID为%s的设备不存在", order.getDeviceId());

    }

    @Override
    public void validate(OrderEndBO bo, boolean ignoreInParking) {
        OrderVO order = bo.getOrder();
        ServiceUtil.assertion(order == null, "参数错误，订单不存在");
        ServiceUtil.assertion(!OrderStatus.canEnd().contains(order.getStatus()), "ID为%s的订单状态不允许结束", order.getId());

        DeviceVO device = bo.getDevice();
        ServiceUtil.assertion(device == null, "参数错误，设备不存在");

        AreaVO area = bo.getArea();
        ServiceUtil.assertion(area == null, "参数错误，运营区不存在");

        // 还车校验
        OrderInParkingVO inParkingVO = bo.getInParkingVO();
        if (!ignoreInParking) {
            ServiceUtil.assertion(inParkingVO == null, "参数错误，停车点信息不存在");
            // 强制在停车点还车
            ServiceUtil.assertion(!inParkingVO.getInParking() && area.getParkingReturn() != null && area.getParkingReturn(), "请将车辆停放在停车点内");
            // 强制在运营区内还车
            ServiceUtil.assertion(!inParkingVO.getInArea() && area.getAreaOutReturn() != null && !area.getAreaOutReturn(), "请将车辆停放在运营区内");
        }
    }

    @Override
    public boolean canOpenDevice(OrderVO order, Long userId) {
        return isUser(order, userId);
    }

    @Override
    public boolean canCloseDevice(OrderVO order, Long userId) {
        return isUser(order, userId);
    }
    
}
